import hashlib
import hmac
import six
import attr
import json


def validate_hub_signature(app_secret, request_payload, hub_signature_header):
    """
        @inputs:
            app_secret: Secret Key for application
            request_payload: request body
            hub_signature_header: X-Hub-Signature header sent with request
        @outputs:
            boolean indicated that hub signature is validated
    """
    try:
        hash_method, hub_signature = hub_signature_header.split('=')
    except:
        pass
    else:
        digest_module = getattr(hashlib, hash_method)
        hmac_object = hmac.new(
            str(app_secret), unicode(request_payload), digest_module)
        generated_hash = hmac_object.hexdigest()
        if hub_signature == generated_hash:
            return True
    return False


def generate_appsecret_proof(access_token, app_secret):
    """
        @inputs:
            access_token: page access token
            app_secret_token: app secret key
        @outputs:
            appsecret_proof: HMAC-SHA256 hash of page access token
                using app_secret as the key
    """
    if six.PY2:
        hmac_object = hmac.new(
            str(app_secret), unicode(access_token), hashlib.sha256)
    else:
        hmac_object = hmac.new(
            bytearray(app_secret, 'utf8'),
            str(access_token).encode('utf8'), hashlib.sha256)
    generated_hash = hmac_object.hexdigest()
    return generated_hash


class AttrsEncoder(json.JSONEncoder):
    def default(self, obj):
        if hasattr(obj, '__attrs_attrs__'):
            items_iterator = (attr.asdict(obj).items()
                              if six.PY3 else
                              attr.asdict(obj).iteritems())
            return {k: v for k, v in items_iterator if v is not None}
        return json.JSONEncoder.default(self, obj)



import attr


@attr.s
class PostbackButton(object):
    """
    See https://developers.facebook.com/docs/messenger-platform/send-api-reference/postback-button
    """
    title = attr.ib()
    payload = attr.ib(default=None)
    type = attr.ib(default='postback')

    def __attrs_post_init__(self):
        assert self.type == 'postback', 'Type of a button can\'t be set ' \
                                        'manually.'
        if not self.payload:
            self.payload = self.title


@attr.s
class CallButton(object):
    """
    See https://developers.facebook.com/docs/messenger-platform/send-api-reference/postback-button
    """
    title = attr.ib()
    payload = attr.ib(convert=str)
    type = attr.ib(default='phone_number')

    def __attrs_post_init__(self):
        assert self.type == 'phone_number', 'Type of a button can\'t be set ' \
                                            'manually.'

        self.payload = self.payload.replace(' ', '')
        assert self.payload.startswith('+'), 'Payload must be a phone number ' \
                                             'with a valid country code.'
        assert self.payload[1:].isnumeric(), \
            'Payload must be a phone number.'


@attr.s
class URLButton(object):
    """
    See https://developers.facebook.com/docs/messenger-platform/send-api-reference/url-button
    """
    title = attr.ib()
    url = attr.ib()
    webview_height_ratio = attr.ib(default='full')
    messenger_extensions = attr.ib(default=None)
    fallback_url = attr.ib(default=None)
    type = attr.ib(default='web_url')

    def __attrs_post_init__(self):
        assert self.type == 'web_url', 'Type of a button can\'t be set ' \
                                       'manually.'


@attr.s
class ShareButton(object):
    """
    See https://developers.facebook.com/docs/messenger-platform/send-api-reference/share-button
    """
    type = attr.ib(default='element_share')

    def __attrs_post_init__(self):
        assert self.type == 'element_share', 'Type of a button can\'t be set ' \
                                             'manually.'
